package jp.co.sds_corp.util.window;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.math.BigDecimal;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

public class ValuePanel extends JPanel
{
	public enum ReturnType {
		STRING,
		INT,
		DOUBLE,
		BIGDECIMAL;
	}
	public enum Type {
		TEXT,
		COMBO,
		LIST,
		PASSWORD;
	}

	ReturnType nRet;
	Type nType;
	JLabel	LBL_KEY = new JLabel();
	JTextField TXT_VAL = new JTextField();
	JComboBox combo;
	XYLayout xyLayout = new XYLayout( 300, 24 );

	public ValuePanel( ReturnType nRet, Type nType, String lbl ) {
		this.nRet = nRet;
		this.nType = nType;
		if ( nType == Type.PASSWORD ) {
			TXT_VAL = new JPasswordField();
		}
		Font	f = new Font( "Dialog", Font.PLAIN, 14 );
		setFont( f );
		setLayout( xyLayout );
		LBL_KEY.setText( lbl );
		add( LBL_KEY, new XYConstraints( 0, 0, 80, 22 ));
		add( TXT_VAL, new XYConstraints( 80, 0, 200, 22 ));

		TextFocusHandler tf = new TextFocusHandler();
		TXT_VAL.addFocusListener( tf );
	}

	public ValuePanel( ReturnType nRet, String lbl ) {
		this( nRet, Type.TEXT, lbl );
	}

	public ValuePanel( ReturnType nRet, String lbl, JComboBox combo ) {
		this.nRet = nRet;
		this.nType = Type.COMBO;
		this.combo = combo;
		Font	f = new Font( "Dialog", Font.PLAIN, 14 );
		setFont( f );
		setLayout( xyLayout );
		LBL_KEY.setText( lbl );
		add( LBL_KEY, new XYConstraints( 0, 0, 80, 22 ));
		add( combo, new XYConstraints( 80, 0, 200, 22 ));
	}

	class TextFocusHandler extends FocusAdapter
	{
		public void focusGained( FocusEvent ev ) {
			JTextField txt = (JTextField ) ev.getSource();
			txt.selectAll();
		}
	}

	public void setKeySize( Dimension dim ) {
		LBL_KEY.setPreferredSize( dim );
	}
	public void setValueSize( Dimension dim ) {
		if ( nType == Type.COMBO ) {
			combo.setPreferredSize( dim );
		} else {
			TXT_VAL.setPreferredSize( dim );
		}
	}

	public void setValue( Object value ) {
		if ( nType == Type.COMBO ) {
			combo.setSelectedItem( value );
		} else {
			TXT_VAL.setText( (String )value );
			TXT_VAL.setCaretPosition( 0 );
		}
	}
	public Object getValue() {
		String val;
		if ( nType == Type.COMBO ) {
			val = (String )combo.getSelectedItem();
		} else {
			val = TXT_VAL.getText();
		}
		switch ( nRet ) {
		case INT:
			try {
				Integer ret = Integer.valueOf( val );
				return ret;
			} catch ( Exception ex ) {
				return null;
			}
		case DOUBLE:
			try {
				Double ret = Double.valueOf( val );
				return ret;
			} catch ( Exception ex ) {
				return null;
			}
		case BIGDECIMAL:
			try {
				BigDecimal ret = new BigDecimal( val );
				return ret;
			} catch ( Exception ex ) {
				return null;
			}
		default:
			return val;
		}
	}

	public void setEnabled( boolean var ) {
		super.setEnabled( var );
		LBL_KEY.setEnabled( var );
		if ( nType == Type.COMBO ) {
			combo.setEnabled( var );
		} else {
			TXT_VAL.setEnabled( var );
		}
	}

	public void setEditable( boolean var ) {
		if ( nType == Type.COMBO ) {
			combo.setEditable( var );
		} else {
			TXT_VAL.setEditable( var );
		}
	}

	public void requestFocus() {
		if ( nType == Type.COMBO ) {
			combo.requestFocus();
		} else {
			TXT_VAL.requestFocus();
		}
	}

}

